# Generated by Django 3.2.20 on 2023-09-14 09:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0035_auto_20230907_1154'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisesubsidybudget',
            name='subsidy_access_policy_display_name',
            field=models.CharField(max_length=512, null=True),
        ),
    ]
